
local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Options")

----------------------------
-- Locals

local ACD = LibStub("AceConfigDialog-3.0")
local LoD = RW:GetPlugin("LoD")
local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")
local LP = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")
local LSM = LibStub("LibSharedMedia-3.0")
local UIF

local iconCoords = {
	STAR		= {0, 		0.25,	0, 		0.25},
	CIRCLE 		= {0.25,	0.5,	0,		0.25},
	DIAMOND		= {0.5, 	0.75,	0,		0.25},
	TRIANGLE 	= {0.75,	1,		0,		0.25},
	MOON 		= {0,		0.25,	0.25,	0.5},
	SQUARE 		= {0.25,	0.5,	0.25,	0.5},
	CROSS 		= {0.5,		0.75,	0.25,	0.5},
	SKULL 		= {0.75,	1,		0.25,	0.5},
}

local OptionsFrame
local DetailsFrame
local OptionsTree
local bosses
local bossMods
local bossModsById
local orderedBossMods
local modIcons

local AceOptionData = {}
local GUIOptionsData = {}
local GUIBossMods = {}
local GUIBossSubGroups = {}

local defaults = {
	profile = {
		VersionSort = "name",
		TestOpt = true,
		Color = {1, 0, 0, 1},
	}
}

----------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Options", defaults)
	
	local sgsg = {
		s = {
			font = LSM:Fetch("font", "accid"),
			fontSize = 16,
		}
	}
	
	UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW", sgsg)
	UIF:SetValuesChangedCallback(function()
		plugin:SendMessage("RW_GUIValuesChanged")
	end)
	RWEW = UIF
	
	self.BossOptions = {BossMods = {}}
	self.PluginOptions = {Plugins = {}}
		
	bosses = {}
	bossMods = {}
	orderedBossMods = {}
	bossModsById = {}
	modIcons = {}
	
	self:RegisterMessage("RW_BossModFound")
	self:RegisterMessage("RW_BossRegistered")
	self:RegisterMessage("RW_GUIValuesChanged")
	self:RegisterMessage("RW_ModLoaded")
	
	self.options = {
		type = "group",
		name = "Raid Watch 2",
		handler = self,
		args = {			
		}
	}	
end

function plugin:PluginEnable()
	self.NextConfigMode = true
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RW", self.options)
	
	self:RegisterMessage("RW_BossRegistered")
	self:RegisterMessage("RW_BossModFound")
	
	for modName, modID in pairs(RW.modStorage) do
		bossMods[modName] = modID
		bossModsById[modID] = modName
		if not RW:InTable(orderedBossMods, modName) then
			tinsert(orderedBossMods, modName)
		end
	end
	
	for name, icon in pairs(RW.iconStorage) do
		modIcons[name] = icon
	end
	
	for i, boss in pairs(RW.bossStorage) do
		bosses[boss.modPack] = bosses[boss.modPack] or {}
		if not RW:InTable(bosses[boss.modPack], boss) then
			tinsert(bosses[boss.modPack], boss)
		end
	end
end

function plugin:PluginDisable()
end

function plugin:ToggleConfigMode()
	RW:SendMessage("RW_ConfigMode", self.NextConfigMode) 
	self.NextConfigMode = not self.NextConfigMode
end

----------------------
-- Message handlers --
----------------------
function plugin:RW_BossRegistered(message, boss)
	bosses[boss.modPack] = bosses[boss.modPack] or {}
	tinsert(bosses[boss.modPack], boss)
end

function plugin:RW_BossModFound(message, modName, modID)
	bossMods[modName] = modID or true
	bossModsById[modID] = modName
	if not RW:InTable(orderedBossMods, modName) then
		tinsert(orderedBossMods, modName)
	end
end

function plugin:RW_ModLoaded(message, modID, modName)
	GUIBossMods[RW:GetIndexForvalue(GUIBossMods, modID)] = nil
	plugin:InsertBossOptions(modName)
	plugin:SendMessage("RW_GUIValuesChanged")
end
----------------------

do
	local treeSelectPath = {}
	local treeStatusTable = {}
	local treeLastSelected	
	
	local bossList = {}
	local bossListLastSelected = {}
	local bossHeaders = {}
	local addedOptions = {}	
	
	------------------------------
	-- Option Details
	
	local function showDetailTab(tabs, event, group)
		tabs:ReleaseChildren()
		local baseObj = tabs:GetUserData("baseObj")
		
		local scroll = AceGUI:Create("ScrollFrame")
		scroll:SetLayout("Flow")	
		
		local added = false
		for i, obj in pairs(baseObj.mod.AlertObjects[group]) do
			if obj.base == baseObj then
				added = true
				scroll:AddChildren(obj:GetOption())
			end
		end
		tabs:AddChild(scroll)
	end
	
	-- Returns the tabs for the details about the selected option
	local tabs = {}
	local function getOptionTabs(baseObj)
		wipe(tabs)
		for i, b in pairs(RW.BITS) do
			if baseObj.mod.AlertObjects[i] then
				local c = false
				for _, obj in pairs(baseObj.mod.AlertObjects[i]) do
					if obj.base == baseObj then
						c = true
						break
					end
				end
				if c then 
					tinsert(tabs, {value = i, text = L[i]})
				end
			end
		end
		return tabs
	end
	
	local function showHideDetails(baseObj)
		if DetailsFrame then 
			DetailsFrame:Release()
			DetailsFrame = nil
		end
			
		local optText = type(baseObj.id) == "number" and RW.SpellName[baseObj.id] or baseObj.id
			
		DetailsFrame = UIF:Frame(optText)
		DetailsFrame:SetLayout("Fill")
		DetailsFrame.frame:ClearAllPoints()
		DetailsFrame.frame:SetPoint("LEFT", OptionsFrame.frame, "RIGHT", 5, 0)
		DetailsFrame:SetWidth(500)
		DetailsFrame:SetHeight(450)
		DetailsFrame:SetCallback("OnClose", function(widget)
			widget:ReleaseChildren()
			widget:Release()
			DetailsFrame = nil
		end)
		
		local t = getOptionTabs(baseObj)
		local tab = UIF:TabGroup()
		tab:SetLayout("Fill")
		tab:SetTabs(t)
		tab:SetUserData("baseObj", baseObj)
		tab:SetCallback("OnGroupSelected", showDetailTab)
		tab:SelectTab(t[1] and t[1].value or nil)
		tab:SetCallback("OnTabEnter", function(widget, event, key, tab)
				GameTooltip:SetOwner(widget.frame)
				GameTooltip:AddLine(L[key])
				GameTooltip:AddLine(L[key.."_DESC"])
				GameTooltip:Show()
			end)
		tab:SetCallback("OnTabLeave", function() GameTooltip:Hide() end)
		
		DetailsFrame:AddChild(tab)
		
		DetailsFrame:Show()
	end
	
	------------------------------
	-- Boss Detials
	
	local function getMasterValue(baseObj)
		if baseObj.type == "bool" then
			return baseObj.mod.DB[baseObj.var]
		else
			return baseObj.mod.DB[baseObj.id].enabled
		end
	end
	
	local function toggleMaster(check, event, value)
		local baseObj = check:GetUserData("baseObj")
		
		if baseObj.type == "bool" then
			baseObj.mod.DB[baseObj.var] = not baseObj.mod.DB[baseObj.var]
		else
			baseObj.mod.DB[baseObj.id].enabled = not baseObj.mod.DB[baseObj.id].enabled
		end
		
		if DetailsFrame then
			showHideDetails(baseObj)
		end
	end
	
	local function GetOptionGroup(opt, boss)
		local baseObj = boss:GetBaseAlertObject(opt)
		--[===[@debug@
		if not baseObj then
			--error(("%s: No option connected to %q"):format(boss:GetName(), opt))
		end
		--@end-debug@]===]		
		
		local optText = type(opt) == "number" and RW.SpellName[opt] or opt
	
		local group = AceGUI:Create("SimpleGroup") -- UIF:InlineGroup2() -- 
		group:SetLayout("Flow")
		group:SetFullWidth(true)
		--group:SetRelativeWidth(0.5)
		--group:SetAutoAdjustHeight(false)
		--group:SetHeight(80)		
		
		if baseObj then
		
			local check = AceGUI:Create("CheckBox")
			check:SetLabel("")
			check:SetUserData("baseObj", baseObj)
			check:SetValue(getMasterValue(baseObj))
			check:SetCallback("OnValueChanged", toggleMaster)
			check:SetRelativeWidth(0.08)
			group:AddChild(check)
		
			local label = UIF:NormalText("", nil, nil, nil, nil, true)
			label:SetImage(baseObj.icon, 0.08, 0.92, 0.08, 0.92)
			label:SetImageSize(25, 25)
			label:SetFullWidth(false)
			label:SetRelativeWidth(0.12)
			label:SetUserData("baseObj", baseObj)
			
			if baseObj.type ~= "bool" then
				label:SetCallback("OnClick", function(self, event, button)
					local baseObj = self:GetUserData("baseObj")
					if button == "LeftButton" then
						if IsShiftKeyDown() and ChatFrameEditBox:IsShown() then
							if baseObj.spell then
								ChatFrameEditBox:Insert("\124cff71d5ff\124Hspell:"..baseObj.spell.."\124h["..RW.SpellName[baseObj.spell].."]\124h\124r")
							end
						end
					end
				end)
			end
			
			if baseObj.spell then
				label:SetCallback("OnEnter", function(self)
					GameTooltip:SetOwner(self.frame)
					GameTooltip:SetHyperlink("spell:"..baseObj.spell)
					GameTooltip:AddLine(" ")
					GameTooltip:AddLine("[Shift + Left click to add link in chat]")
					GameTooltip:Show()
				end)
				label:SetCallback("OnLeave", function()
					GameTooltip:Hide()
				end)
			end			
			group:AddChild(label)
			
			label = UIF:NormalText(optText)
			label:SetFullWidth(false)
			label:SetRelativeWidth(0.29)
			group:AddChild(label)			
			
			local iconGroup = AceGUI:Create("SimpleGroup")
			iconGroup:SetLayout("Flow")
			iconGroup:SetFullWidth(false)
			iconGroup:SetRelativeWidth(0.3)
			group:AddChild(iconGroup)
			
			local iconObj = boss:GetIconObject(opt)
			if iconObj then
				
				for i in pairs(iconObj.icons) do
					local icon = AceGUI:Create("Icon")
					icon:SetImage("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp", unpack(iconCoords[i]))
					icon:SetImageSize(18, 18)
					icon:SetHeight(20)
					icon:SetWidth(25)
					iconGroup:AddChild(icon)
				end
			end
			
			local button = UIF:Button("Details", function(widget, event)
				local baseObj = widget:GetUserData("baseObj")
				showHideDetails(baseObj)
			end)
			button:SetRelativeWidth(0.2)
			button:SetUserData("baseObj", baseObj)
			group:AddChild(button)
		end
		
		return group
	end
	
	local function selectBoss(drop, event, group)
		drop:ReleaseChildren()
		
		local boss = drop:GetUserData("bosses")[group]
		drop:SetUserData("boss", boss)
		
		bossListLastSelected[drop:GetUserData("bosses")] = group
		
		local scroll = AceGUI:Create("ScrollFrame")
		drop:AddChild(scroll)
		
		local sgroup = UIF:InlineGroup(" ")
		scroll:AddChild(sgroup)
		
		local label = UIF:MainTitleLabel(boss.L and boss.L.name or boss:GetName())
		label:SetImage(boss.BossIcon)
		sgroup:AddChild(label)
		
		if rawget(boss.L, "info") then
			label = UIF:NormalText(rawget(boss.L, "info"))		
			sgroup:AddChild(label)
		end
		
		local enableCheck = UIF:CheckBox("Enabled", boss.db.profile, "enabled")
		enableCheck:SetHeight(35)
		sgroup:AddChild(enableCheck)
		
		if boss.Options then
			-- Build options titles
			if boss.Options.Categories then
				for i, cat in pairs(boss.Options.Categories) do
					local group = UIF:InlineGroup(cat.text)
					group:SetLayout("Flow")
					scroll:AddChild(group)
					
					label = UIF:NormalText("Enabled", nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.1)
					group:AddChild(label)
					
					label = UIF:NormalText("Icon", nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.1)
					group:AddChild(label)
					
					label = UIF:NormalText("Name", nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.29)
					group:AddChild(label)
					
					label = UIF:NormalText("Raid Icons", nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.3)
					group:AddChild(label)
					
					label = UIF:NormalText("Details", nil, nil, nil, "OUTLINE")
					label:SetRelativeWidth(0.2)
					group:AddChild(label)
					
					for j, opt in ipairs(cat) do
						local optGroup = GetOptionGroup(opt, boss)
						if optGroup then
							group:AddChildren(optGroup)
						end
					end
				end
			end			
		end		
		
		scroll:FixScroll()
	end
	
	------------------------------
	-- Boss Selection
			
	local function selectTreeOption(tree, event, group)
		treeLastSelected = group
		tree:ReleaseChildren()
		tree:PauseLayout()
		if AceOptionData[group] then
			local subGroups
			for _, child in pairs(AceOptionData[group].args) do
				if child.type == "group" and not child.guiInline then
					subGroups = true
					break
				end
			end
			if subGroups then
			
				addFrame = AceGUI:Create("SimpleGroup")
				addFrame:SetLayout("Fill")
				addFrame.width = "fill"
				addFrame.height = "fill"
				
				ACD:Open("RW", addFrame, string.split("\001", group))
				tree:AddChild(addFrame)
			else
				local scroll = AceGUI:Create("ScrollFrame")
				scroll:SetLayout("flow")
				scroll.width = "fill"
				scroll.height = "fill"
				
				local addFrame = AceGUI:Create("SimpleGroup")
				addFrame:SetLayout("Fill")
				addFrame.width = "fill"
				addFrame.height = "fill"
				ACD:Open("RW", addFrame, string.split("\001", group))
				scroll:AddChild(addFrame)
				tree:AddChild(scroll)
			end
			
		elseif GUIOptionsData[group] then
			tree:AddChildren(GUIOptionsData[group]())

		elseif GUIBossMods[group] then
			if type(GUIBossMods[group]) == "table" then
				local nameList
				for i, boss in pairs(GUIBossMods[group]) do
					nameList = nameList or {}
					nameList[i] = {value=i, text=(boss.L and boss.L.name or boss:GetName()), icon = boss.BossIcon}
				end
				
				dropdown = UIF:DropdownGroup("test")
				dropdown:SetDropdownWidth(300)
				dropdown:SetLayout("Fill")
				dropdown:SetUserData("bosses", GUIBossMods[group])
				dropdown:SetFullHeight(true)
				dropdown:SetGroupList(nameList)
				dropdown:SetCallback("OnGroupSelected", selectBoss)
				if bossListLastSelected[GUIBossMods[group]] and nameList[bossListLastSelected[GUIBossMods[group]]] then
					dropdown:SetGroup(bossListLastSelected[GUIBossMods[group]])
				else
					dropdown:SetGroup(1)
				end
				tree:AddChild(dropdown)
				
			else
				local modID = GUIBossMods[group]
				local modName = bossModsById[modID]
				
				local sgroup = AceGUI:Create("SimpleGroup")
				sgroup:SetLayout("Flow")
				
				local button = UIF:Button("Load "..modName, function(widget, event)
					LoD:LoadMod(widget:GetUserData("modID"), widget:GetUserData("modName"))
				end)
				button:SetUserData("modID", modID)
				button:SetUserData("modName", modName)
				sgroup:AddChild(button)
				
				tree:AddChild(sgroup)				
			end		
		elseif GUIBossSubGroups[group] then
			local sgroup = AceGUI:Create("ScrollFrame")
			sgroup:SetLayout("Flow")
			tree:AddChild(sgroup)
			
			for i, data in pairs(GUIBossSubGroups[group]) do
				local label = UIF:MainTitleLabel(data.name)
				label:SetImage(data.icon)
				sgroup:AddChild(label)
			end
			
			sgroup:FixScroll()
		else
			--print(string.split("\001", group))
		end	
		
		tree:ResumeLayout()
		tree:DoLayout()
	end
	-----------------------------
	-- Main window
	
	function plugin:Toggle()
		if OptionsFrame then
			OptionsFrame:Release()
			OptionsFrame = nil
		else	
			OptionsFrame = UIF:Frame("Raid Watch 2")
			OptionsFrame:SetLayout("Fill")
			OptionsFrame:SetWidth(810)
			OptionsFrame:SetHeight(620)
			OptionsFrame:SetCallback("OnClose", function(widget)
				widget:ReleaseChildren()
				widget:Release()
				if DetailsFrame then
					DetailsFrame:Release()
					DetailsFrame = nil
				end
				OptionsFrame = nil
			end)
			
			OptionsTree = UIF:TreeGroup()
			OptionsTree:SetLayout("Fill")
			OptionsTree:SetFullHeight(true)
			OptionsTree:SetTree(self:BuildOptionsTree())
			OptionsTree:SetCallback("OnGroupSelected", selectTreeOption)		
			OptionsTree:SetSelected(treeLastSelected or "0Main")	
			OptionsTree:SetStatusTable(treeStatusTable)
			OptionsFrame:AddChild(OptionsTree)
			
			OptionsFrame:Show()
			AceGUI:ClearFocus()
		end
	end
	
	function plugin:RW_GUIValuesChanged()
		if OptionsFrame then			
			OptionsTree:SetTree(self:BuildOptionsTree())
			local last = treeLastSelected
			OptionsTree:SetSelected("0Main")			
			OptionsTree:SetSelected(last or "0Main")
		end
	end
end

do
	local Tree
	
	local catIcons2 = {
		["Alert Plugins"] = [[Interface\Icons\INV_Sigil_UlduarAll]],
		["Misc"] = [[Interface\Icons\Achievement_Boss_CThun]],
	}
	
	local predefinedOrders = {
		["Plugins"] = 3
	}
	
	local function BuildAceOptions(data, insertPoint, ...)
		local nextOpt = ...
		
		--if not nextOpt then return insertPoint end		
					
		if select(2, ...) then				
			insertPoint[nextOpt] = insertPoint[nextOpt] or {
				type = "group",
				name = nextOpt,
				args = {},
			}
			BuildAceOptions(data, insertPoint[nextOpt].args, select(2, ...))
		else 
			insertPoint[nextOpt] = data
		end
	end
	
	function plugin:BuildPluginOptions(insertPoint, module)
		local optionID
		local ordered
		local order
					
		for i, data in pairs(module) do
			insertPoint.children = insertPoint.children or {}
			
			if type(data) == "number" and not ordered then
				order = data
				ordered = true
			end
			
			if type(data) == "string" then		
				if not RW:InTable2(insertPoint.children, "text", data) then						
					local newCat = {text = data, value = data, icon = catIcons2[data]}
					tinsert(insertPoint.children, newCat)
					if order then
						newCat.value = order..newCat.value
						order = nil
					end
					if predefinedOrders[data] then
						newCat.value = predefinedOrders[data]..newCat.value
					end
					table.sort(insertPoint.children or insertPoint, function(a, b) return a.value < b.value end)
					insertPoint = newCat						
					optionID = (optionID and optionID.."\001" or "")..newCat.value
				else
					local id = data
					if order then
						id = order..id
						order = nil
					end
					if predefinedOrders[data] then
						id = predefinedOrders[data]..id
					end
					optionID = (optionID and optionID.."\001" or "")..id
					insertPoint = RW:GetTableData(insertPoint.children, "text", data)
				end								
			end
				
			if type(data) == "table" then					
				insertPoint.icon = insertPoint.icon or data.icon
				if #insertPoint.children == 0 then insertPoint.children = nil end
				AceOptionData[optionID] = data
				BuildAceOptions(data, self.options.args, string.split("\001", optionID))											
			end
			
			if type(data) == "function" then
				if #insertPoint.children == 0 then insertPoint.children = nil end
				insertPoint.icon = insertPoint.icon or module[i + 1]
				GUIOptionsData[optionID] = data
				table.sort(insertPoint.children or insertPoint, function(a, b) return a.value < b.value end)	
				break
			end	

			table.sort(insertPoint.children or insertPoint, function(a, b) return a.value < b.value end)					
		end
	end		
	
	local function BuildBossModOptions(insertPoint, modName)
		insertPoint.children = insertPoint.children or {}
		local optionID = "1Boss Mods\001"..modName
		local modID = bossMods[modName]
		local newCat
		if not RW:InTable2(insertPoint.children, "text", modName) then
			newCat = {value = modName, text = modName, icon = modIcons[modName]}		
			tinsert(insertPoint.children, newCat)
		else
			newCat = RW:GetTableData(insertPoint.children, "text", modName)
		end
		if bosses[modName] then
			for i, boss in pairs(bosses[modName]) do
				newCat.icon = newCat.icon or boss.icon
				if boss.subGroup then
					newCat.children = newCat.children or {}
					if not RW:InTable2(newCat.children, "text", boss.subGroup) then
						local child = {value = boss.subGroup, text = boss.subGroup, icon = boss.SubCatIcon}
						tinsert(newCat.children, child)
						GUIBossSubGroups[optionID] = GUIBossSubGroups[optionID] or {}
						tinsert(GUIBossSubGroups[optionID], {name = boss.subGroup, icon = boss.SubCatIcon})
					end
					local oldID = optionID
					optionID = optionID.."\001"..boss.subGroup
					GUIBossMods[optionID] = GUIBossMods[optionID] or {}	
					tinsert(GUIBossMods[optionID] , boss)
					optionID = oldID				
				else
					GUIBossMods[optionID] = GUIBossMods[optionID] or {}
					tinsert(GUIBossMods[optionID] , boss)
				end								 
			end				
		else
			GUIBossMods[optionID] = modID 
		end
	end
	
	function plugin:BuildOptionsTree(rebuild)		
		
		if not Tree then
			Tree = {}
			for _, module in RW:IteratePlugins() do
				if module.GetOptions then	
					local mOpts = {module:GetOptions()}
					if type(mOpts[1]) == "table" then
						for i, opts in pairs(mOpts[1]) do
							self:BuildPluginOptions(Tree, opts)
						end
					else
						self:BuildPluginOptions(Tree, mOpts)
					end
				end
			end
			Tree.children = Tree.children or {}
			local newCat
			if not RW:InTable2(Tree.children, "text", "Boss Mods") then
				newCat = {value = "1Boss Mods", text = "Boss Mods"}
				tinsert(Tree.children, newCat)
			else
				newCat = RW:GetTableData(Tree.children, "text", "Boss Mods")
			end
			for _, modName in pairs(orderedBossMods) do
				BuildBossModOptions(newCat, modName)
			end
		end
		
		table.sort(Tree.children, function(a, b) return a.value < b.value end)	
		
		return Tree.children
	end
	
	function plugin:InsertBossOptions(modName)
		local bossTree = RW:GetTableData(Tree.children, "text", "Boss Mods")
		local modTree = RW:GetTableData(bossTree.children, "text", modName)
		BuildBossModOptions(bossTree, modName)
	end
end

-- GUI
function plugin:GetOptions()
	return {
		{0, "Main", self.GetGUI1}, 
		{"Plugins", self.GetGUI2},
		{8, "Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(RW.db)},
		{"Plugins", "Alert Plugins", self.GetAlertDesc},
		{"Plugins", "Misc", self.GetMiscDesc},
	}
end

do		
	local MainTitleColor = {0.4, 0.6, 0.8}
	local SectionTitelColor = {18/255, 157/255, 143/255}
	local ListColor = {0.96, 0.93, 0.49}
	local NormalTextColor = {177/255, 219/255, 255/255}
	
	local MainTitleSize = 35
	local SectionTitelSize = 20
	local NormalTextSize = 16
	
	function plugin:GetGUI1()
		local self = self or plugin
		
		local group, button, label
		
		group = AceGUI:Create("ScrollFrame")
		group:SetLayout("Flow")
		group:PauseLayout()
		
		label = UIF:MainTitleLabel("Raid Watch 2")
		group:AddChild(label)
		
		igroup = UIF:InlineGroup("")
		group:AddChild(igroup)	
		
		label = UIF:NormalText("Welcome and thankyou for you using Raid Watch 2. This addon will help you in boss enounters troughout the game in several ways, timers for important event, cooldown of things, warnings and much more. Please look trough the options and customize it to the way you want it to be used.")
		label:SetRelativeWidth(0.8)
		igroup:AddChild(label)	
		
		label = UIF:ListText(" ")
		igroup:AddChild(label)
		
		label = UIF:Level1Title("Features")
		igroup:AddChild(label)		
		
		label = UIF:ListText(
[[
   |cffffffff*|r Timers
   |cffffffff*|r Warnings
   |cffffffff*|r Messages
   |cffffffff*|r Automatic raid marking
   |cffffffff*|r Range check
   |cffffffff*|r Health frames
   |cffffffff*|r Filter out spamm
   |cffffffff*|r Automatic whisper replies
]])
		label:SetRelativeWidth(1)
		igroup:AddChild(label)
		
		label = UIF:Level1Title("How to configure:")
		igroup:AddChild(label)
		
		label = UIF:NormalText(
[[
To enter configuration mode please select plugins in the list to the left and then click the button |cffffff77"Toggle Config Mode"|r. Once in configuration mode, anchors for each plugin (if it has any) will show up and you can drag it in place where you want it.

You can also go into the individual plugins settings to configure them in more detail, also in the list to the left.w
]])
		label:SetRelativeWidth(0.8)
		igroup:AddChild(label)
		
		button = UIF:Button("To Plugins...", function()
			OptionsTree:SetSelected("3Plugins")
		end)
		button:SetFullWidth(true)
		igroup:AddChild(button)
		
				
		group:ResumeLayout()
		group:DoLayout()
		
		group:FixScroll()
		
		return group
	end

	function plugin:GetGUI2()
		local self = self or plugin
		
		local group, button, label
		
		group = AceGUI:Create("ScrollFrame")
		group:SetLayout("Flow")
		
		button = UIF:Button("Toggle Config Mode", function()
			RW:SendMessage("RW_ConfigMode", self.NextConfigMode) 
			self.NextConfigMode = not self.NextConfigMode
		end)
		group:AddChild(button)
		
		return group
	end
	
	function plugin:GetAlertDesc()
		local self = self or plugin
		
		local group, button, label
		
		group = AceGUI:Create("ScrollFrame")
		group:SetLayout("Flow")
		
		label = UIF:NormalText(LP.ALERT_PLUGINS_DESC)
		label:SetImage([[Interface\Icons\INV_Sigil_UlduarAll]])
		group:AddChild(label)
		
		return group
	end

	function plugin:GetMiscDesc()
		local self = self or plugin
		
		local group, button, label
		
		group = AceGUI:Create("ScrollFrame")
		group:SetLayout("Flow")
		
		label = UIF:NormalText(LP.MISC_PLUGINS_DESC)
		label:SetImage([[Interface\Icons\Achievement_Boss_CThun]])
		group:AddChild(label)
		
		return group
	end
end

--------------------------
-- Option helpers

plugin.OptionHelpers = {}
function plugin.OptionHelpers.OnOptionEnter(self, event)
	local obj = self:GetUserData("obj")
	GameTooltip:SetOwner(self.frame)
	GameTooltip:AddLine(L[obj.mainType.."_"..obj.type])
	GameTooltip:AddLine(L[obj.mainType.."_"..obj.type.."_DESC"]:format(obj.base.id))
	GameTooltip:Show()
end

function plugin.OptionHelpers.OnOptionLeave(self, event)
	GameTooltip:Hide()
end

function plugin.OptionHelpers.OnColorSelected(self, event, r, g, b, a)
	local obj = self:GetUserData("obj")
	local db = obj.base.mod.DB
	db[obj.base.id][obj.mainType][obj.optText or obj.type].color = {r, g, b, a}
end

function plugin.OptionHelpers.OnValueChanged(self, event, value)
	local obj = self:GetUserData("obj")
	local db = obj.base.mod.DB
	db[obj.base.id][obj.mainType][obj.optText or obj.type].enabled = value
end

function plugin.OptionHelpers.OnKeyValueChanged(self, event, value)
	local obj, key = self:GetUserData("obj"), self:GetUserData("key")
	local db = obj.base.mod.DB
	db[obj.base.id][obj.mainType][obj.optText or obj.type][key] = value
end