--[[
Plugin holds default settings (font, colors, textures and more)
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("DefaultSettings")

----------------------------------
-- Locals
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")

local db

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace(self:GetName(), RW.defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

--[[ function plugin:PluginEnable()
end

function plugin:PluginDisable()
end ]]--

function plugin:Reset()
	db = self.db.profile
	for i in pairs(db) do
		self:SendMessage("RW_DefaultsChanged", i, db[i])
	end
end

----------------------------------
-- Option creators

function plugin:GetOptions()
	return "Plugins", "Misc", "Global Settings", self.GetGUI, [[Interface\Icons\Achievement_Dungeon_Ulduar77_25man]]
end

local function changeCallback(widget, event, ...)
	local key = widget:GetUserData("key")
	if key then
		plugin:SendMessage("RW_DefaultsChanged", key, db[key])
	end
end

local function changeCallback2(widget, event, ...)
	local key = widget:GetUserData("key")
	if key then	
		if select("#", ...) > 1 then
			for i = 1, select("#", ...) do
				db[key][i] = select(i, ...)
			end
		else
			db[key] = ...
		end
		
		plugin:SendMessage("RW_DefaultsChanged", key, db[key])
	end
end

function plugin:GetGUI()
	local self = self or plugin
	local UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	local group, label, slider, check, igroup, button, color, select
	
	group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
	
	label = UIF:MainTitleLabel("Global Settings")
	group:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	label = UIF:NormalText(L.GLOBAL_SETTINGS_DESC)
	label:SetImage([[Interface\Icons\Achievement_Dungeon_Ulduar77_25man]])
	igroup:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	local sgroup = UIF:InlineGroup2("Font")
	igroup:AddChild(sgroup)
	
	select = AceGUI:Create("LSM30_Font")
	select:SetLabel("Font")
	select:SetList()
	select:SetValue(db["Font"])
	select:SetUserData("db", db)
	select:SetUserData("key", "Font")
	select:SetCallback("OnValueChanged", changeCallback2)
	sgroup:AddChild(select)
	
	slider = UIF:Slider(L.FONT_SIZE, db, "FontSize", 5, 50, 1, changeCallback)
	sgroup:AddChild(slider)
	
	color = UIF:ColorSelect(L.FONT_COLOR, db, "FontColor", changeCallback)
	sgroup:AddChild(color)
	
	
	select = UIF:Dropdown(L.FONT_OUTLINE)
	select:SetList({OUTLINE = "Outline", THICKOUTLINE = "Thick Outline", NONE = "None"})
	select:SetValue(db["FontOutline"])
	select:SetUserData("db", db)
	select:SetUserData("key", "FontOutline")
	select:SetCallback("OnValueChanged", changeCallback2)
	sgroup:AddChild(select)
		
	sgroup = UIF:InlineGroup2(L.TEXTURE_MAIN)
	igroup:AddChild(sgroup)
	
	select = AceGUI:Create("LSM30_Border")
	select:SetLabel("Border")
	select:SetList()
	select:SetValue(db["Border"])
	select:SetUserData("db", db)
	select:SetUserData("key", "Border")
	select:SetCallback("OnValueChanged", changeCallback2)
	sgroup:AddChild(select)
	
	color = UIF:ColorSelect(L.BORDER_COLOR, db, "BorderColor", changeCallback)
	sgroup:AddChild(color)
	
	slider = UIF:Slider(L.BORDER_SIZE, db, "BorderSize", 5, 40, 1, changeCallback)
	sgroup:AddChild(slider)		
	
	select = AceGUI:Create("LSM30_Statusbar")
	select:SetLabel(L.BAR_TEXTURE)
	select:SetList()
	select:SetValue(db["BarTexture"])
	select:SetUserData("db", db)
	select:SetUserData("key", "BarTexture")
	select:SetCallback("OnValueChanged", changeCallback2)
	sgroup:AddChild(select)
				
	color = UIF:ColorSelect(L.BAR_COLOR, db, "BarColor", changeCallback)
	sgroup:AddChild(color)
		
	button = UIF:Button(L.RESET_SETTINGS, function() 
		plugin.db:ResetProfile() 
	end)
	button:SetUserData("confirm", true)
	button:SetUserData("TriggerGlobalUpdate", true)
	group:AddChild(button)
	
	group:FixScroll()
	
	return group
end
